# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for AI Platform Tensorboard commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.ai.tensorboard_time_series import client
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import resources


def ParseTensorboardOperation(operation_name):
  """Parse operation relative resource name to the operation reference object.

  Args:
    operation_name: The operation resource name

  Returns:
    The operation reference object
  """
  collection = 'aiplatform.projects.locations'
  if '/tensorboards/' in operation_name:
    collection += '.tensorboards'
  if '/experiments/' in operation_name:
    collection += '.experiments'
  if '/runs/' in operation_name:
    collection += '.runs'
  collection += '.operations'
  try:
    return resources.REGISTRY.ParseRelativeName(
        operation_name, collection=collection)
  except resources.WrongResourceCollectionException:
    return resources.REGISTRY.ParseRelativeName(
        operation_name, collection='aiplatform.projects.locations.operations')


_TYPE_CHOICES = {
    'SCALAR': (
        'scalar',
        'Used for tensorboard-time-series that is a list of scalars. E.g. '
        'accuracy of a model over epochs/time.'
    ),
    'TENSOR': (
        'tensor',
        'Used for tensorboard-time-series that is a list of tensors. E.g. '
        'histograms of weights of layer in a model over epoch/time.'
    ),
    'BLOB_SEQUENCE': (
        'blob-sequence',
        'Used for tensorboard-time-series that is a list of blob sequences. '
        'E.g. set of sample images with labels over epochs/time.'
    ),
}


def GetTensorboardTimeSeriesTypeArg(noun):
  return arg_utils.ChoiceEnumMapper(
      '--type',
      client.GetMessagesModule(
      ).GoogleCloudAiplatformV1beta1TensorboardTimeSeries
      .ValueTypeValueValuesEnum,
      required=True,
      custom_mappings=_TYPE_CHOICES,
      help_str='Value type of the {noun}.'.format(noun=noun),
      default=None)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for canceling executions."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core.console import console_io


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Cancel(base.Command):
  """Cancel an execution."""

  detailed_help = {
      'DESCRIPTION':
          """
          {description}
          """,
      'EXAMPLES':
          """
          To cancel an execution:

              $ {command} my-execution
          """,
  }

  @staticmethod
  def CommonArgs(parser):
    execution_presentation = presentation_specs.ResourcePresentationSpec(
        'EXECUTION',
        resource_args.GetExecutionResourceSpec(),
        'Execution to cancel.',
        required=True,
        prefixes=False)
    concept_parsers.ConceptParser([execution_presentation]).AddToParser(parser)

  @staticmethod
  def Args(parser):
    Cancel.CommonArgs(parser)

  def Run(self, args):
    """Cancel an execution."""
    conn_context = connection_context.GetConnectionContext(
        args, flags.Product.RUN, self.ReleaseTrack())
    ex_ref = args.CONCEPTS.execution.Parse()

    console_io.PromptContinue(
        message='Execution [{}] will be canceled.'.format(ex_ref.executionsId),
        throw_if_unattended=True,
        cancel_on_no=True)

    with serverless_operations.Connect(conn_context) as client:
      client.CancelExecution(ex_ref)
    pretty_print.Success('Canceled execution [{}].'.format(ex_ref.executionsId))



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Eventarc Channels API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.eventarc import common
from googlecloudsdk.api_lib.eventarc.base import EventarcClientBase
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties


class NoFieldsSpecifiedError(exceptions.Error):
  """Error when no fields were specified for a Patch operation."""


def GetProject(args):
  """Gets project resource from either argument flag or attribute."""
  return args.project or properties.VALUES.core.project.GetOrFail()


def GetLocation(args):
  """Gets location resource from either argument flag or attribute."""
  return args.location or properties.VALUES.eventarc.location.GetOrFail()


class GoogleChannelConfigClientV1(EventarcClientBase):
  """Google Channel Client for interaction with v1 of Eventarc Channels API."""

  def __init__(self):
    super(GoogleChannelConfigClientV1,
          self).__init__(common.API_NAME, common.API_VERSION_1,
                         'GoogleChannelConfig')

    # Eventarc Client
    client = apis.GetClientInstance(common.API_NAME, common.API_VERSION_1)

    self._messages = client.MESSAGES_MODULE
    self._service = client.projects_locations

  def Get(self, google_channel_config_name):
    """Gets the requested GoogleChannelConfig.

    Args:
      google_channel_config_name: str, the name of GoogleChannelConfig to get.

    Returns:
      The GoogleChannelConfig message.
    """
    get_req = self._messages.EventarcProjectsLocationsGetGoogleChannelConfigRequest(
        name=google_channel_config_name)
    return self._service.GetGoogleChannelConfig(get_req)

  def Update(self, google_channel_config_name, google_channel_config_message,
             update_mask):
    """Updates the specified Channel.

    Args:
      google_channel_config_name: str, the name of GoogleChannelConfig to
        update.
      google_channel_config_message: GoogleChannelConfig, the config message
        that holds KMS encryption info.
      update_mask: str, a comma-separated list of GoogleChannelConfig fields to
        update.

    Returns:
      Response for update.
    """
    update_req = self._messages.EventarcProjectsLocationsUpdateGoogleChannelConfigRequest(
        name=google_channel_config_name,
        googleChannelConfig=google_channel_config_message,
        updateMask=update_mask)
    return self._service.UpdateGoogleChannelConfig(update_req)

  def BuildGoogleChannelConfig(self, google_channel_config_name,
                               crypto_key_name):
    return self._messages.GoogleChannelConfig(
        name=google_channel_config_name, cryptoKeyName=crypto_key_name)

  def BuildUpdateMask(self, crypto_key, clear_crypto_key):
    """Builds an update mask for updating a channel.

    Args:
      crypto_key: bool, whether to update the crypto key.
      clear_crypto_key: bool, whether to clear the crypto key.

    Returns:
      The update mask as a string.

    Raises:
      NoFieldsSpecifiedError: No fields are being updated.
    """
    update_mask = []
    if crypto_key:
      update_mask.append('cryptoKeyName')
    if clear_crypto_key:
      update_mask.append('cryptoKeyName')

    if not update_mask:
      raise NoFieldsSpecifiedError('Must specify at least one field to update.')
    return ','.join(update_mask)
